// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
// SPDX-FileCopyrightText: 2024 Harald Sitter <sitter@kde.org>

#include "platformdependent_v3.h"

Attica::PlatformDependentV3::~PlatformDependentV3() = default;

#include "moc_platformdependent_v3.cpp"
